@@ -11,7 +11,7 @@
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with
- TALER; see the file COPYING.  If not, If not, see <http://www.gnu.org/licenses/>
+ TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
 import {AmountJson} from "./types";
@@ -36,11 +36,12 @@ const GNUNET_SYSERR = -1;
 
 let Module = EmscWrapper.Module;
 
-let getEmsc: EmscWrapper.EmscFunGen = (...args) => Module.cwrap.apply(null,
-                                                                      args);
+let getEmsc: EmscWrapper.EmscFunGen = (...args: any[]) => Module.cwrap.apply(
+  null,
+  args);
 
 var emsc = {
-  free: (ptr) => Module._free(ptr),
+  free: (ptr: number) => Module._free(ptr),
   get_value: getEmsc('TALER_WR_get_value',
                      'number',
                      ['number']),
@@ -164,13 +165,12 @@ enum RandomQuality {
 
 
 abstract class ArenaObject {
-  private _nativePtr: number;
+  protected _nativePtr: number | undefined = undefined;
   arena: Arena;
 
   abstract destroy(): void;
 
   constructor(arena?: Arena) {
-    this.nativePtr = null;
     if (!arena) {
       if (arenaStack.length == 0) {
         throw Error("No arena available")
@@ -192,14 +192,14 @@ abstract class ArenaObject {
   }
 
   free() {
-    if (this.nativePtr !== undefined) {
+    if (this.nativePtr) {
       emsc.free(this.nativePtr);
-      this.nativePtr = undefined;
+      this._nativePtr = undefined;
     }
   }
 
   alloc(size: number) {
-    if (this.nativePtr !== undefined) {
+    if (this._nativePtr !== undefined) {
       throw Error("Double allocation");
     }
     this.nativePtr = emscAlloc.malloc(size);
@@ -212,21 +212,22 @@ abstract class ArenaObject {
     this._nativePtr = n;
   }
 
-  set nativePtr(v) {
+  set nativePtr(v: number) {
     this.setNative(v);
   }
 
   get nativePtr() {
     return this.getNative();
   }
-
 }
 
+
 interface Arena {
   put(obj: ArenaObject): void;
   destroy(): void;
 }
 
+
 class DefaultArena implements Arena {
   heap: Array<ArenaObject>;
 
@@ -234,7 +235,7 @@ class DefaultArena implements Arena {
     this.heap = [];
   }
 
-  put(obj) {
+  put(obj: ArenaObject) {
     this.heap.push(obj);
   }
 
@@ -269,7 +270,7 @@ class SyncArena extends DefaultArena {
     super();
   }
 
-  pub(obj) {
+  pub(obj: ArenaObject) {
     super.put(obj);
     if (!this.isScheduled) {
       this.schedule();
@@ -308,14 +309,12 @@ export class Amount extends ArenaObject {
   }
 
   destroy() {
-    if (this.nativePtr != 0) {
-      emsc.free(this.nativePtr);
-    }
+    super.free();
   }
 
 
   static getZero(currency: string, a?: Arena): Amount {
-    let am = new Amount(null, a);
+    let am = new Amount(undefined, a);
     let r = emsc.amount_get_zero(currency, am.getNative());
     if (r != GNUNET_OK) {
       throw Error("invalid currency");
@@ -397,6 +396,30 @@ export class Amount extends ArenaObject {
 }
 
 
+/**
+ * Count the UTF-8 characters in a JavaScript string.
+ */
+function countBytes(str: string): number {
+  var s = str.length;
+  // JavaScript strings are UTF-16 arrays
+  for (let i = str.length - 1; i >= 0; i--) {
+    var code = str.charCodeAt(i);
+    if (code > 0x7f && code <= 0x7ff) {
+      // We need an extra byte in utf-8 here
+      s++;
+    } else if (code > 0x7ff && code <= 0xffff) {
+      // We need two extra bytes in utf-8 here
+      s += 2;
+    }
+    // Skip over the other surrogate
+    if (code >= 0xDC00 && code <= 0xDFFF) {
+      i--;
+    }
+  }
+  return s;
+}
+
+
 /**
  * Managed reference to a contiguous block of memory in the Emscripten heap.
  * Should contain only data, not pointers.
@@ -442,7 +465,8 @@ abstract class PackedArenaObject extends ArenaObject {
   }
 
   alloc() {
-    if (this.nativePtr === null) {
+    // FIXME: should the client be allowed to call alloc multiple times?
+    if (!this._nativePtr) {
       this.nativePtr = emscAlloc.malloc(this.size());
     }
   }
@@ -466,7 +490,7 @@ abstract class PackedArenaObject extends ArenaObject {
       b = (b + 256) % 256;
       bytes.push("0".concat(b.toString(16)).slice(-2));
     }
-    let lines = [];
+    let lines: string[] = [];
     for (let i = 0; i < bytes.length; i += 8) {
       lines.push(bytes.slice(i, i + 8).join(","));
     }
@@ -482,7 +506,7 @@ export class AmountNbo extends PackedArenaObject {
 
   toJson(): any {
     let a = new DefaultArena();
-    let am = new Amount(null, a);
+    let am = new Amount(undefined, a);
     am.fromNbo(this);
     let json = am.toJson();
     a.destroy();
@@ -508,7 +532,7 @@ export class EddsaPrivateKey extends PackedArenaObject {
     return obj;
   }
 
-  static fromCrock: (string) => EddsaPrivateKey;
+  static fromCrock: (s: string) => EddsaPrivateKey;
 }
 mixinStatic(EddsaPrivateKey, fromCrock);
 
@@ -521,7 +545,7 @@ function fromCrock(s: string) {
 }
 
 
-function mixin(obj, method, name?: string) {
+function mixin(obj: any, method: any, name?: string) {
   if (!name) {
     name = method.name;
   }
@@ -532,7 +556,7 @@ function mixin(obj, method, name?: string) {
 }
 
 
-function mixinStatic(obj, method, name?: string) {
+function mixinStatic(obj: any, method: any, name?: string) {
   if (!name) {
     name = method.name;
   }
@@ -595,7 +619,7 @@ export class RsaBlindingKeySecret extends PackedArenaObject {
     return o;
   }
 
-  static fromCrock: (string) => RsaBlindingKeySecret;
+  static fromCrock: (s: string) => RsaBlindingKeySecret;
 }
 mixinStatic(RsaBlindingKeySecret, fromCrock);
 
@@ -622,9 +646,9 @@ export class ByteArray extends PackedArenaObject {
     return this.allocatedSize;
   }
 
-  constructor(desiredSize: number, init: number, a?: Arena) {
+  constructor(desiredSize: number, init?: number, a?: Arena) {
     super(a);
-    if (init === undefined || init === null) {
+    if (init === undefined) {
       this.nativePtr = emscAlloc.malloc(desiredSize);
     } else {
       this.nativePtr = init;
@@ -633,17 +657,20 @@ export class ByteArray extends PackedArenaObject {
   }
 
   static fromString(s: string, a?: Arena): ByteArray {
-    let hstr = emscAlloc.malloc(s.length + 1);
-    Module.writeStringToMemory(s, hstr);
-    return new ByteArray(s.length, hstr, a);
+    // UTF-8 bytes, including 0-terminator
+    let terminatedByteLength = countBytes(s) + 1;
+    let hstr = emscAlloc.malloc(terminatedByteLength);
+    Module.stringToUTF8(s, hstr, terminatedByteLength);
+    return new ByteArray(terminatedByteLength, hstr, a);
   }
 
   static fromCrock(s: string, a?: Arena): ByteArray {
-    let hstr = emscAlloc.malloc(s.length + 1);
-    Module.writeStringToMemory(s, hstr);
-    let decodedLen = Math.floor((s.length * 5) / 8);
-    let ba = new ByteArray(decodedLen, null, a);
-    let res = emsc.string_to_data(hstr, s.length, ba.nativePtr, decodedLen);
+    let byteLength = countBytes(s);
+    let hstr = emscAlloc.malloc(byteLength + 1);
+    Module.stringToUTF8(s, hstr, byteLength + 1);
+    let decodedLen = Math.floor((byteLength * 5) / 8);
+    let ba = new ByteArray(decodedLen, undefined, a);
+    let res = emsc.string_to_data(hstr, byteLength, ba.nativePtr, decodedLen);
     emsc.free(hstr);
     if (res != GNUNET_OK) {
       throw Error("decoding failed");
@@ -777,7 +804,7 @@ export class AbsoluteTimeNbo extends PackedArenaObject {
     x.alloc();
     let r = /Date\(([0-9]+)\)/;
     let m = r.exec(s);
-    if (m.length != 2) {
+    if (!m || m.length != 2) {
       throw Error();
     }
     let n = parseInt(m[1]) * 1000000;
@@ -899,11 +926,11 @@ interface Encodeable {
   encode(arena?: Arena): ByteArray;
 }
 
-function makeEncode(encodeFn) {
+function makeEncode(encodeFn: any) {
   function encode(arena?: Arena) {
     let ptr = emscAlloc.malloc(PTR_SIZE);
     let len = encodeFn(this.getNative(), ptr);
-    let res = new ByteArray(len, null, arena);
+    let res = new ByteArray(len, undefined, arena);
     res.setNative(Module.getValue(ptr, '*'));
     emsc.free(ptr);
     return res;
